==> jei-1.20.1-fabric-15.20.0.128-sources.jar: mezz/jei/library/plugins/jei/info/IngredientInfoRecipe.java <==
package mezz.jei.library.plugins.jei.info;

import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.ingredients.ITypedIngredient;
import mezz.jei.api.recipe.vanilla.IJeiIngredientInfoRecipe;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.util.StringUtil;
import mezz.jei.library.ingredients.TypedIngredient;
import net.minecraft.class_2561;
import net.minecraft.class_5348;
import java.util.List;
import java.util.Optional;

public class IngredientInfoRecipe implements IJeiIngredientInfoRecipe {
	private final List<class_5348> description;
	private final List<ITypedIngredient<?>> ingredients;

	public static <T> IJeiIngredientInfoRecipe create(
		IIngredientManager ingredientManager,
		List<T> ingredients,
		IIngredientType<T> ingredientType,
		class_2561... descriptionComponents
	) {
		List<ITypedIngredient<?>> typedIngredients = ingredients.stream()
			.map(i -> TypedIngredient.createAndFilterInvalid(ingredientManager, ingredientType, i, true))
			.<ITypedIngredient<?>>flatMap(Optional::stream)
			.toList();

		List<class_5348> descriptionLines = StringUtil.expandNewlines(descriptionComponents);
		return new IngredientInfoRecipe(typedIngredients, descriptionLines);
	}

	private IngredientInfoRecipe(List<ITypedIngredient<?>> ingredients, List<class_5348> description) {
		this.description = description;
		this.ingredients = ingredients;
	}

	@Override
	public List<class_5348> getDescription() {
		return description;
	}

	@Override
	public List<ITypedIngredient<?>> getIngredients() {
		return ingredients;
	}
}

==> jei-1.21.11-fabric-27.3.0.12-sources.jar: mezz/jei/library/plugins/jei/info/IngredientInfoRecipe.java <==
package mezz.jei.library.plugins.jei.info;

import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.ingredients.ITypedIngredient;
import mezz.jei.api.recipe.vanilla.IJeiIngredientInfoRecipe;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.util.StringUtil;
import mezz.jei.library.ingredients.TypedIngredient;
import net.minecraft.class_2561;
import net.minecraft.class_5348;
import java.util.Collections;
import java.util.List;

public class IngredientInfoRecipe implements IJeiIngredientInfoRecipe {
	private final List<class_5348> description;
	private final List<ITypedIngredient<?>> ingredients;

	public static <T> IJeiIngredientInfoRecipe create(
		IIngredientManager ingredientManager,
		List<T> ingredients,
		IIngredientType<T> ingredientType,
		class_2561... descriptionComponents
	) {
		List<ITypedIngredient<T>> typedIngredients = TypedIngredient.createAndFilterInvalidNonnullList(ingredientManager, ingredientType, ingredients, true);
		List<class_5348> descriptionLines = StringUtil.expandNewlines(descriptionComponents);
		return new IngredientInfoRecipe(typedIngredients, descriptionLines);
	}

	private IngredientInfoRecipe(List<? extends ITypedIngredient<?>> ingredients, List<class_5348> description) {
		this.description = description;
		this.ingredients = Collections.unmodifiableList(ingredients);
	}

	@Override
	public List<class_5348> getDescription() {
		return description;
	}

	@Override
	public List<ITypedIngredient<?>> getIngredients() {
		return ingredients;
	}
}
